@@ -13,7 +13,6 @@
 #include "TaskScheduler.h"
 #include "draw.h"
 #include "sample.h"
-#include "settings.h"
 
 #include "box2d/base.h"
 #include "box2d/box2d.h"
@@ -53,10 +52,9 @@ static int MyAllocHook( int allocType, void* userData, size_t size, int blockTyp
 }
 #endif
 
-GLFWwindow* g_mainWindow = nullptr;
+static SampleContext s_context;
 static int32_t s_selection = 0;
 static Sample* s_sample = nullptr;
-static Settings s_settings;
 static bool s_rightMouseDown = false;
 static b2Vec2 s_clickPointWS = b2Vec2_zero;
 static float s_windowScale = 1.0f;
@@ -75,7 +73,7 @@ void* AllocFcn( uint32_t size, int32_t alignment )
 	size_t sizeAligned = ( ( size - 1 ) | ( alignment - 1 ) ) + 1;
 	assert( ( sizeAligned & ( alignment - 1 ) ) == 0 );
 
-#if defined( _MSC_VER )
+#if defined( _MSC_VER ) || defined( __MINGW32__ ) || defined( __MINGW64__ )
 	void* ptr = _aligned_malloc( sizeAligned, alignment );
 #else
 	void* ptr = aligned_alloc( alignment, sizeAligned );
@@ -86,7 +84,7 @@ void* AllocFcn( uint32_t size, int32_t alignment )
 
 void FreeFcn( void* mem )
 {
-#if defined( _MSC_VER )
+#if defined( _MSC_VER ) || defined( __MINGW32__ ) || defined( __MINGW64__ )
 	_aligned_free( mem );
 #else
 	free( mem );
@@ -127,9 +125,9 @@ static void RestartSample()
 {
 	delete s_sample;
 	s_sample = nullptr;
-	s_settings.restart = true;
-	s_sample = g_sampleEntries[s_settings.sampleIndex].createFcn( s_settings );
-	s_settings.restart = false;
+	s_context.restart = true;
+	s_sample = g_sampleEntries[s_context.sampleIndex].createFcn( &s_context );
+	s_context.restart = false;
 }
 
 static void CreateUI( GLFWwindow* window, const char* glslVersion )
@@ -158,10 +156,10 @@ static void CreateUI( GLFWwindow* window, const char* glslVersion )
 	{
 		ImFontConfig fontConfig;
 		fontConfig.RasterizerMultiply = s_windowScale * s_framebufferScale;
-		g_draw.m_smallFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 14.0f, &fontConfig );
-		g_draw.m_regularFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 18.0f, &fontConfig );
-		g_draw.m_mediumFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 40.0f, &fontConfig );
-		g_draw.m_largeFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 64.0f, &fontConfig );
+		s_context.draw.m_smallFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 14.0f, &fontConfig );
+		s_context.draw.m_regularFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 18.0f, &fontConfig );
+		s_context.draw.m_mediumFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 40.0f, &fontConfig );
+		s_context.draw.m_largeFont = ImGui::GetIO().Fonts->AddFontFromFileTTF( fontPath, 64.0f, &fontConfig );
 	}
 	else
 	{
@@ -179,10 +177,8 @@ static void DestroyUI()
 
 static void ResizeWindowCallback( GLFWwindow*, int width, int height )
 {
-	g_camera.m_width = int( width / s_windowScale );
-	g_camera.m_height = int( height / s_windowScale );
-	s_settings.windowWidth = int( width / s_windowScale );
-	s_settings.windowHeight = int( height / s_windowScale );
+	s_context.camera.m_width = int( width / s_windowScale );
+	s_context.camera.m_height = int( height / s_windowScale );
 }
 
 static void KeyCallback( GLFWwindow* window, int key, int scancode, int action, int mods )
@@ -199,7 +195,7 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 		{
 			case GLFW_KEY_ESCAPE:
 				// Quit
-				glfwSetWindowShouldClose( g_mainWindow, GL_TRUE );
+				glfwSetWindowShouldClose( s_context.window, GL_TRUE );
 				break;
 
 			case GLFW_KEY_LEFT:
@@ -211,7 +207,7 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 				}
 				else
 				{
-					g_camera.m_center.x -= 0.5f;
+					s_context.camera.m_center.x -= 0.5f;
 				}
 				break;
 
@@ -224,7 +220,7 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 				}
 				else
 				{
-					g_camera.m_center.x += 0.5f;
+					s_context.camera.m_center.x += 0.5f;
 				}
 				break;
 
@@ -237,7 +233,7 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 				}
 				else
 				{
-					g_camera.m_center.y -= 0.5f;
+					s_context.camera.m_center.y -= 0.5f;
 				}
 				break;
 
@@ -250,12 +246,12 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 				}
 				else
 				{
-					g_camera.m_center.y += 0.5f;
+					s_context.camera.m_center.y += 0.5f;
 				}
 				break;
 
 			case GLFW_KEY_HOME:
-				g_camera.ResetView();
+				s_context.camera.ResetView();
 				break;
 
 			case GLFW_KEY_R:
@@ -263,11 +259,11 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 				break;
 
 			case GLFW_KEY_O:
-				s_settings.singleStep = true;
+				s_context.singleStep = true;
 				break;
 
 			case GLFW_KEY_P:
-				s_settings.pause = !s_settings.pause;
+				s_context.pause = !s_context.pause;
 				break;
 
 			case GLFW_KEY_LEFT_BRACKET:
@@ -289,7 +285,7 @@ static void KeyCallback( GLFWwindow* window, int key, int scancode, int action,
 				break;
 
 			case GLFW_KEY_TAB:
-				g_draw.m_showUI = !g_draw.m_showUI;
+				s_context.draw.m_showUI = !s_context.draw.m_showUI;
 
 			default:
 				if ( s_sample )
@@ -305,9 +301,9 @@ static void CharCallback( GLFWwindow* window, unsigned int c )
 	ImGui_ImplGlfw_CharCallback( window, c );
 }
 
-static void MouseButtonCallback( GLFWwindow* window, int32_t button, int32_t action, int32_t mods )
+static void MouseButtonCallback( GLFWwindow* window, int button, int action, int modifiers )
 {
-	ImGui_ImplGlfw_MouseButtonCallback( window, button, action, mods );
+	ImGui_ImplGlfw_MouseButtonCallback( window, button, action, modifiers );
 
 	if ( ImGui::GetIO().WantCaptureMouse )
 	{
@@ -315,16 +311,16 @@ static void MouseButtonCallback( GLFWwindow* window, int32_t button, int32_t act
 	}
 
 	double xd, yd;
-	glfwGetCursorPos( g_mainWindow, &xd, &yd );
+	glfwGetCursorPos( window, &xd, &yd );
 	b2Vec2 ps = { float( xd ) / s_windowScale, float( yd ) / s_windowScale };
 
 	// Use the mouse to move things around.
 	if ( button == GLFW_MOUSE_BUTTON_1 )
 	{
-		b2Vec2 pw = g_camera.ConvertScreenToWorld( ps );
+		b2Vec2 pw = s_context.camera.ConvertScreenToWorld( ps );
 		if ( action == GLFW_PRESS )
 		{
-			s_sample->MouseDown( pw, button, mods );
+			s_sample->MouseDown( pw, button, modifiers );
 		}
 
 		if ( action == GLFW_RELEASE )
@@ -336,7 +332,7 @@ static void MouseButtonCallback( GLFWwindow* window, int32_t button, int32_t act
 	{
 		if ( action == GLFW_PRESS )
 		{
-			s_clickPointWS = g_camera.ConvertScreenToWorld( ps );
+			s_clickPointWS = s_context.camera.ConvertScreenToWorld( ps );
 			s_rightMouseDown = true;
 		}
 
@@ -353,15 +349,15 @@ static void MouseMotionCallback( GLFWwindow* window, double xd, double yd )
 
 	ImGui_ImplGlfw_CursorPosCallback( window, ps.x, ps.y );
 
-	b2Vec2 pw = g_camera.ConvertScreenToWorld( ps );
+	b2Vec2 pw = s_context.camera.ConvertScreenToWorld( ps );
 	s_sample->MouseMove( pw );
 
 	if ( s_rightMouseDown )
 	{
 		b2Vec2 diff = b2Sub( pw, s_clickPointWS );
-		g_camera.m_center.x -= diff.x;
-		g_camera.m_center.y -= diff.y;
-		s_clickPointWS = g_camera.ConvertScreenToWorld( ps );
+		s_context.camera.m_center.x -= diff.x;
+		s_context.camera.m_center.y -= diff.y;
+		s_clickPointWS = s_context.camera.ConvertScreenToWorld( ps );
 	}
 }
 
@@ -375,11 +371,11 @@ static void ScrollCallback( GLFWwindow* window, double dx, double dy )
 
 	if ( dy > 0 )
 	{
-		g_camera.m_zoom /= 1.1f;
+		s_context.camera.m_zoom /= 1.1f;
 	}
 	else
 	{
-		g_camera.m_zoom *= 1.1f;
+		s_context.camera.m_zoom *= 1.1f;
 	}
 }
 
@@ -388,12 +384,12 @@ static void UpdateUI()
 	int maxWorkers = enki::GetNumHardwareThreads();
 
 	float menuWidth = 180.0f;
-	if ( g_draw.m_showUI )
+	if ( s_context.draw.m_showUI )
 	{
-		ImGui::SetNextWindowPos( { g_camera.m_width - menuWidth - 10.0f, 10.0f } );
-		ImGui::SetNextWindowSize( { menuWidth, g_camera.m_height - 20.0f } );
+		ImGui::SetNextWindowPos( { s_context.camera.m_width - menuWidth - 10.0f, 10.0f } );
+		ImGui::SetNextWindowSize( { menuWidth, s_context.camera.m_height - 20.0f } );
 
-		ImGui::Begin( "Tools", &g_draw.m_showUI,
+		ImGui::Begin( "Tools", &s_context.draw.m_showUI,
 					  ImGuiWindowFlags_NoMove | ImGuiWindowFlags_NoResize | ImGuiWindowFlags_NoCollapse );
 
 		if ( ImGui::BeginTabBar( "ControlTabs", ImGuiTabBarFlags_None ) )
@@ -401,49 +397,49 @@ static void UpdateUI()
 			if ( ImGui::BeginTabItem( "Controls" ) )
 			{
 				ImGui::PushItemWidth( 100.0f );
-				ImGui::SliderInt( "Sub-steps", &s_settings.subStepCount, 1, 50 );
-				ImGui::SliderFloat( "Hertz", &s_settings.hertz, 5.0f, 120.0f, "%.0f hz" );
+				ImGui::SliderInt( "Sub-steps", &s_context.subStepCount, 1, 32 );
+				ImGui::SliderFloat( "Hertz", &s_context.hertz, 5.0f, 240.0f, "%.0f hz" );
 
-				if ( ImGui::SliderInt( "Workers", &s_settings.workerCount, 1, maxWorkers ) )
+				if ( ImGui::SliderInt( "Workers", &s_context.workerCount, 1, maxWorkers ) )
 				{
-					s_settings.workerCount = b2ClampInt( s_settings.workerCount, 1, maxWorkers );
+					s_context.workerCount = b2ClampInt( s_context.workerCount, 1, maxWorkers );
 					RestartSample();
 				}
 				ImGui::PopItemWidth();
 
 				ImGui::Separator();
 
-				ImGui::Checkbox( "Sleep", &s_settings.enableSleep );
-				ImGui::Checkbox( "Warm Starting", &s_settings.enableWarmStarting );
-				ImGui::Checkbox( "Continuous", &s_settings.enableContinuous );
+				ImGui::Checkbox( "Sleep", &s_context.enableSleep );
+				ImGui::Checkbox( "Warm Starting", &s_context.enableWarmStarting );
+				ImGui::Checkbox( "Continuous", &s_context.enableContinuous );
 
 				ImGui::Separator();
 
-				ImGui::Checkbox( "Shapes", &s_settings.drawShapes );
-				ImGui::Checkbox( "Joints", &s_settings.drawJoints );
-				ImGui::Checkbox( "Joint Extras", &s_settings.drawJointExtras );
-				ImGui::Checkbox( "Bounds", &s_settings.drawBounds );
-				ImGui::Checkbox( "Contact Points", &s_settings.drawContactPoints );
-				ImGui::Checkbox( "Contact Normals", &s_settings.drawContactNormals );
-				ImGui::Checkbox( "Contact Impulses", &s_settings.drawContactImpulses );
-				ImGui::Checkbox( "Contact Features", &s_settings.drawContactFeatures );
-				ImGui::Checkbox( "Friction Impulses", &s_settings.drawFrictionImpulses );
-				ImGui::Checkbox( "Mass", &s_settings.drawMass );
-				ImGui::Checkbox( "Body Names", &s_settings.drawBodyNames );
-				ImGui::Checkbox( "Graph Colors", &s_settings.drawGraphColors );
-				ImGui::Checkbox( "Islands", &s_settings.drawIslands );
-				ImGui::Checkbox( "Counters", &s_settings.drawCounters );
-				ImGui::Checkbox( "Profile", &s_settings.drawProfile );
+				ImGui::Checkbox( "Shapes", &s_context.drawShapes );
+				ImGui::Checkbox( "Joints", &s_context.drawJoints );
+				ImGui::Checkbox( "Joint Extras", &s_context.drawJointExtras );
+				ImGui::Checkbox( "Bounds", &s_context.drawBounds );
+				ImGui::Checkbox( "Contact Points", &s_context.drawContactPoints );
+				ImGui::Checkbox( "Contact Normals", &s_context.drawContactNormals );
+				ImGui::Checkbox( "Contact Impulses", &s_context.drawContactImpulses );
+				ImGui::Checkbox( "Contact Features", &s_context.drawContactFeatures );
+				ImGui::Checkbox( "Friction Impulses", &s_context.drawFrictionImpulses );
+				ImGui::Checkbox( "Mass", &s_context.drawMass );
+				ImGui::Checkbox( "Body Names", &s_context.drawBodyNames );
+				ImGui::Checkbox( "Graph Colors", &s_context.drawGraphColors );
+				ImGui::Checkbox( "Islands", &s_context.drawIslands );
+				ImGui::Checkbox( "Counters", &s_context.drawCounters );
+				ImGui::Checkbox( "Profile", &s_context.drawProfile );
 
 				ImVec2 button_sz = ImVec2( -1, 0 );
 				if ( ImGui::Button( "Pause (P)", button_sz ) )
 				{
-					s_settings.pause = !s_settings.pause;
+					s_context.pause = !s_context.pause;
 				}
 
 				if ( ImGui::Button( "Single Step (O)", button_sz ) )
 				{
-					s_settings.singleStep = !s_settings.singleStep;
+					s_context.singleStep = !s_context.singleStep;
 				}
 
 				if ( ImGui::Button( "Dump Mem Stats", button_sz ) )
@@ -463,7 +459,7 @@ static void UpdateUI()
 
 				if ( ImGui::Button( "Quit", button_sz ) )
 				{
-					glfwSetWindowShouldClose( g_mainWindow, GL_TRUE );
+					glfwSetWindowShouldClose( s_context.window, GL_TRUE );
 				}
 
 				ImGui::EndTabItem();
@@ -481,7 +477,7 @@ static void UpdateUI()
 				int i = 0;
 				while ( i < g_sampleCount )
 				{
-					bool categorySelected = strcmp( category, g_sampleEntries[s_settings.sampleIndex].category ) == 0;
+					bool categorySelected = strcmp( category, g_sampleEntries[s_context.sampleIndex].category ) == 0;
 					ImGuiTreeNodeFlags nodeSelectionFlags = categorySelected ? ImGuiTreeNodeFlags_Selected : 0;
 					bool nodeOpen = ImGui::TreeNodeEx( category, nodeFlags | nodeSelectionFlags );
 
@@ -490,7 +486,7 @@ static void UpdateUI()
 						while ( i < g_sampleCount && strcmp( category, g_sampleEntries[i].category ) == 0 )
 						{
 							ImGuiTreeNodeFlags selectionFlags = 0;
-							if ( s_settings.sampleIndex == i )
+							if ( s_context.sampleIndex == i )
 							{
 								selectionFlags = ImGuiTreeNodeFlags_Selected;
 							}
@@ -548,16 +544,13 @@ int main( int, char** )
 
 	char buffer[128];
 
-	s_settings.Load();
-	s_settings.workerCount = b2MinInt( 8, (int)enki::GetNumHardwareThreads() / 2 );
+	s_context.Load();
+	s_context.workerCount = b2MinInt( 8, (int)enki::GetNumHardwareThreads() / 2 );
 
 	SortSamples();
 
 	glfwSetErrorCallback( glfwErrorCallback );
 
-	g_camera.m_width = s_settings.windowWidth;
-	g_camera.m_height = s_settings.windowHeight;
-
 	if ( glfwInit() == 0 )
 	{
 		fprintf( stderr, "Failed to initialize GLFW\n" );
@@ -576,7 +569,7 @@ int main( int, char** )
 	glfwWindowHint( GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE );
 
 	// MSAA
-	//glfwWindowHint( GLFW_SAMPLES, 4 );
+	glfwWindowHint( GLFW_SAMPLES, 4 );
 
 	b2Version version = b2GetVersion();
 	snprintf( buffer, 128, "Box2D Version %d.%d.%d", version.major, version.minor, version.revision );
@@ -593,29 +586,29 @@ int main( int, char** )
 	bool fullscreen = false;
 	if ( fullscreen )
 	{
-		g_mainWindow = glfwCreateWindow( int( 1920 * s_windowScale ), int( 1080 * s_windowScale ), buffer,
-										 glfwGetPrimaryMonitor(), nullptr );
+		s_context.window = glfwCreateWindow( int( 1920 * s_windowScale ), int( 1080 * s_windowScale ), buffer,
+											 glfwGetPrimaryMonitor(), nullptr );
 	}
 	else
 	{
-		g_mainWindow = glfwCreateWindow( int( g_camera.m_width * s_windowScale ), int( g_camera.m_height * s_windowScale ),
-										 buffer, nullptr, nullptr );
+		s_context.window = glfwCreateWindow( int( s_context.camera.m_width * s_windowScale ),
+											 int( s_context.camera.m_height * s_windowScale ), buffer, nullptr, nullptr );
 	}
 
-	if ( g_mainWindow == nullptr )
+	if ( s_context.window == nullptr )
 	{
-		fprintf( stderr, "Failed to open GLFW g_mainWindow.\n" );
+		fprintf( stderr, "Failed to open GLFW window.\n" );
 		glfwTerminate();
 		return -1;
 	}
 
 #ifdef __APPLE__
-	glfwGetWindowContentScale( g_mainWindow, &s_framebufferScale, &s_framebufferScale );
+	glfwGetWindowContentScale( s_context.window, &s_framebufferScale, &s_framebufferScale );
 #else
-	glfwGetWindowContentScale( g_mainWindow, &s_windowScale, &s_windowScale );
+	glfwGetWindowContentScale( s_context.window, &s_windowScale, &s_windowScale );
 #endif
 
-	glfwMakeContextCurrent( g_mainWindow );
+	glfwMakeContextCurrent( s_context.window );
 
 	// Load OpenGL functions using glad
 	if ( !gladLoadGL() )
@@ -626,81 +619,73 @@ int main( int, char** )
 	}
 
 	{
-		//float temp1[2] = {};
-		//glGetFloatv( GL_ALIASED_LINE_WIDTH_RANGE, temp1 );
-
-		//float temp2[2] = {};
-		//glGetFloatv( GL_SMOOTH_LINE_WIDTH_RANGE, temp2 );
-
 		const char* glVersionString = (const char*)glGetString( GL_VERSION );
 		const char* glslVersionString = (const char*)glGetString( GL_SHADING_LANGUAGE_VERSION );
 		printf( "OpenGL %s, GLSL %s\n", glVersionString, glslVersionString );
-		//printf( "OpenGL aliased line width range : [%g, %g]\n", temp1[0], temp1[1] );
-		//printf( "OpenGL smooth line width range : [%g, %g]\n", temp2[0], temp2[1] );
 	}
 
-	glfwSetWindowSizeCallback( g_mainWindow, ResizeWindowCallback );
-	glfwSetKeyCallback( g_mainWindow, KeyCallback );
-	glfwSetCharCallback( g_mainWindow, CharCallback );
-	glfwSetMouseButtonCallback( g_mainWindow, MouseButtonCallback );
-	glfwSetCursorPosCallback( g_mainWindow, MouseMotionCallback );
-	glfwSetScrollCallback( g_mainWindow, ScrollCallback );
+	glfwSetWindowSizeCallback( s_context.window, ResizeWindowCallback );
+	glfwSetKeyCallback( s_context.window, KeyCallback );
+	glfwSetCharCallback( s_context.window, CharCallback );
+	glfwSetMouseButtonCallback( s_context.window, MouseButtonCallback );
+	glfwSetCursorPosCallback( s_context.window, MouseMotionCallback );
+	glfwSetScrollCallback( s_context.window, ScrollCallback );
 
-	// todo put this in s_settings
-	CreateUI( g_mainWindow, glslVersion );
-	g_draw.Create();
+	// todo put this in s_context
+	CreateUI( s_context.window, glslVersion );
+	s_context.draw.Create(&s_context.camera);
 
-	s_settings.sampleIndex = b2ClampInt( s_settings.sampleIndex, 0, g_sampleCount - 1 );
-	s_selection = s_settings.sampleIndex;
+	s_context.sampleIndex = b2ClampInt( s_context.sampleIndex, 0, g_sampleCount - 1 );
+	s_selection = s_context.sampleIndex;
 
 	glClearColor( 0.2f, 0.2f, 0.2f, 1.0f );
 
 	float frameTime = 0.0;
 
-	while ( !glfwWindowShouldClose( g_mainWindow ) )
+	while ( !glfwWindowShouldClose( s_context.window ) )
 	{
 		double time1 = glfwGetTime();
 
-		if ( glfwGetKey( g_mainWindow, GLFW_KEY_Z ) == GLFW_PRESS )
+		if ( glfwGetKey( s_context.window, GLFW_KEY_Z ) == GLFW_PRESS )
 		{
 			// Zoom out
-			g_camera.m_zoom = b2MinFloat( 1.005f * g_camera.m_zoom, 100.0f );
+			s_context.camera.m_zoom = b2MinFloat( 1.005f * s_context.camera.m_zoom, 100.0f );
 		}
-		else if ( glfwGetKey( g_mainWindow, GLFW_KEY_X ) == GLFW_PRESS )
+		else if ( glfwGetKey( s_context.window, GLFW_KEY_X ) == GLFW_PRESS )
 		{
 			// Zoom in
-			g_camera.m_zoom = b2MaxFloat( 0.995f * g_camera.m_zoom, 0.5f );
+			s_context.camera.m_zoom = b2MaxFloat( 0.995f * s_context.camera.m_zoom, 0.5f );
 		}
 
-		glfwGetWindowSize( g_mainWindow, &g_camera.m_width, &g_camera.m_height );
-		g_camera.m_width = int( g_camera.m_width / s_windowScale );
-		g_camera.m_height = int( g_camera.m_height / s_windowScale );
+		glfwGetWindowSize( s_context.window, &s_context.camera.m_width, &s_context.camera.m_height );
+		s_context.camera.m_width = int( s_context.camera.m_width / s_windowScale );
+		s_context.camera.m_height = int( s_context.camera.m_height / s_windowScale );
 
 		int bufferWidth, bufferHeight;
-		glfwGetFramebufferSize( g_mainWindow, &bufferWidth, &bufferHeight );
+		glfwGetFramebufferSize( s_context.window, &bufferWidth, &bufferHeight );
 		glViewport( 0, 0, bufferWidth, bufferHeight );
 
 		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
 
-		// g_draw.DrawBackground();
+		// s_context.draw.DrawBackground();
 
 		double cursorPosX = 0, cursorPosY = 0;
-		glfwGetCursorPos( g_mainWindow, &cursorPosX, &cursorPosY );
-		ImGui_ImplGlfw_CursorPosCallback( g_mainWindow, cursorPosX / s_windowScale, cursorPosY / s_windowScale );
+		glfwGetCursorPos( s_context.window, &cursorPosX, &cursorPosY );
+		ImGui_ImplGlfw_CursorPosCallback( s_context.window, cursorPosX / s_windowScale, cursorPosY / s_windowScale );
 		ImGui_ImplOpenGL3_NewFrame();
 		ImGui_ImplGlfw_NewFrame();
-		ImGui_ImplGlfw_CursorPosCallback( g_mainWindow, cursorPosX / s_windowScale, cursorPosY / s_windowScale );
+		ImGui_ImplGlfw_CursorPosCallback( s_context.window, cursorPosX / s_windowScale, cursorPosY / s_windowScale );
 
 		ImGuiIO& io = ImGui::GetIO();
-		io.DisplaySize.x = float( g_camera.m_width );
-		io.DisplaySize.y = float( g_camera.m_height );
-		io.DisplayFramebufferScale.x = bufferWidth / float( g_camera.m_width );
-		io.DisplayFramebufferScale.y = bufferHeight / float( g_camera.m_height );
+		io.DisplaySize.x = float( s_context.camera.m_width );
+		io.DisplaySize.y = float( s_context.camera.m_height );
+		io.DisplayFramebufferScale.x = bufferWidth / float( s_context.camera.m_width );
+		io.DisplayFramebufferScale.y = bufferHeight / float( s_context.camera.m_height );
 
 		ImGui::NewFrame();
 
 		ImGui::SetNextWindowPos( ImVec2( 0.0f, 0.0f ) );
-		ImGui::SetNextWindowSize( ImVec2( float( g_camera.m_width ), float( g_camera.m_height ) ) );
+		ImGui::SetNextWindowSize( ImVec2( float( s_context.camera.m_width ), float( s_context.camera.m_height ) ) );
 		ImGui::SetNextWindowBgAlpha( 0.0f );
 		ImGui::Begin( "Overlay", nullptr,
 					  ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoInputs | ImGuiWindowFlags_AlwaysAutoResize |
@@ -710,34 +695,34 @@ int main( int, char** )
 		if ( s_sample == nullptr )
 		{
 			// delayed creation because imgui doesn't create fonts until NewFrame() is called
-			s_sample = g_sampleEntries[s_settings.sampleIndex].createFcn( s_settings );
+			s_sample = g_sampleEntries[s_context.sampleIndex].createFcn( &s_context );
 		}
 
-		if ( g_draw.m_showUI )
+		if ( s_context.draw.m_showUI )
 		{
-			const SampleEntry& entry = g_sampleEntries[s_settings.sampleIndex];
+			const SampleEntry& entry = g_sampleEntries[s_context.sampleIndex];
 			snprintf( buffer, 128, "%s : %s", entry.category, entry.name );
 			s_sample->DrawTitle( buffer );
 		}
 
-		s_sample->Step( s_settings );
+		s_sample->Step();
 
-		g_draw.Flush();
+		s_context.draw.Flush();
 
 		UpdateUI();
 
 		// ImGui::ShowDemoWindow();
 
-		if ( g_draw.m_showUI )
+		if ( s_context.draw.m_showUI )
 		{
 			snprintf( buffer, 128, "%.1f ms - step %d - camera (%g, %g, %g)", 1000.0f * frameTime, s_sample->m_stepCount,
-					  g_camera.m_center.x, g_camera.m_center.y, g_camera.m_zoom );
+					  s_context.camera.m_center.x, s_context.camera.m_center.y, s_context.camera.m_zoom );
 			// snprintf( buffer, 128, "%.1f ms", 1000.0f * frameTime );
 
 			ImGui::Begin( "Overlay", nullptr,
 						  ImGuiWindowFlags_NoTitleBar | ImGuiWindowFlags_NoInputs | ImGuiWindowFlags_AlwaysAutoResize |
 							  ImGuiWindowFlags_NoScrollbar );
-			ImGui::SetCursorPos( ImVec2( 5.0f, g_camera.m_height - 20.0f ) );
+			ImGui::SetCursorPos( ImVec2( 5.0f, s_context.camera.m_height - 20.0f ) );
 			ImGui::TextColored( ImColor( 153, 230, 153, 255 ), "%s", buffer );
 			ImGui::End();
 		}
@@ -745,24 +730,24 @@ int main( int, char** )
 		ImGui::Render();
 		ImGui_ImplOpenGL3_RenderDrawData( ImGui::GetDrawData() );
 
-		glfwSwapBuffers( g_mainWindow );
+		glfwSwapBuffers( s_context.window );
 
 		// For the Tracy profiler
 		FrameMark;
 
-		if ( s_selection != s_settings.sampleIndex )
+		if ( s_selection != s_context.sampleIndex )
 		{
-			g_camera.ResetView();
-			s_settings.sampleIndex = s_selection;
+			s_context.camera.ResetView();
+			s_context.sampleIndex = s_selection;
 
 			// #todo restore all drawing settings that may have been overridden by a sample
-			s_settings.subStepCount = 4;
-			s_settings.drawJoints = true;
-			s_settings.useCameraBounds = false;
+			s_context.subStepCount = 4;
+			s_context.drawJoints = true;
+			s_context.useCameraBounds = false;
 
 			delete s_sample;
 			s_sample = nullptr;
-			s_sample = g_sampleEntries[s_settings.sampleIndex].createFcn( s_settings );
+			s_sample = g_sampleEntries[s_context.sampleIndex].createFcn( &s_context );
 		}
 
 		glfwPollEvents();
@@ -782,12 +767,12 @@ int main( int, char** )
 	delete s_sample;
 	s_sample = nullptr;
 
-	g_draw.Destroy();
+	s_context.draw.Destroy();
 
 	DestroyUI();
 	glfwTerminate();
 
-	s_settings.Save();
+	s_context.Save();
 
 #if defined( _MSC_VER )
 	_CrtDumpMemoryLeaks();
