@@ -3,7 +3,6 @@
 
 #include "draw.h"
 #include "sample.h"
-#include "settings.h"
 
 #include "box2d/box2d.h"
 
@@ -15,13 +14,13 @@
 class HighMassRatio1 : public Sample
 {
 public:
-	explicit HighMassRatio1( Settings& settings )
-		: Sample( settings )
+	explicit HighMassRatio1( SampleContext* context )
+		: Sample( context )
 	{
-		if ( settings.restart == false )
+		if ( m_context->restart == false )
 		{
-			g_camera.m_center = { 3.0f, 14.0f };
-			g_camera.m_zoom = 25.0f;
+			m_context->camera.m_center = { 3.0f, 14.0f };
+			m_context->camera.m_zoom = 25.0f;
 		}
 
 		float extent = 1.0f;
@@ -66,9 +65,9 @@ public:
 		}
 	}
 
-	static Sample* Create( Settings& settings )
+	static Sample* Create( SampleContext* context )
 	{
-		return new HighMassRatio1( settings );
+		return new HighMassRatio1( context );
 	}
 };
 
@@ -78,13 +77,13 @@ static int sampleIndex1 = RegisterSample( "Robustness", "HighMassRatio1", HighMa
 class HighMassRatio2 : public Sample
 {
 public:
-	explicit HighMassRatio2( Settings& settings )
-		: Sample( settings )
+	explicit HighMassRatio2( SampleContext* context )
+		: Sample( context )
 	{
-		if ( settings.restart == false )
+		if ( m_context->restart == false )
 		{
-			g_camera.m_center = { 0.0f, 16.5f };
-			g_camera.m_zoom = 25.0f;
+			m_context->camera.m_center = { 0.0f, 16.5f };
+			m_context->camera.m_zoom = 25.0f;
 		}
 
 		{
@@ -124,9 +123,9 @@ public:
 		}
 	}
 
-	static Sample* Create( Settings& settings )
+	static Sample* Create( SampleContext* context )
 	{
-		return new HighMassRatio2( settings );
+		return new HighMassRatio2( context );
 	}
 };
 
@@ -136,13 +135,13 @@ static int sampleIndex2 = RegisterSample( "Robustness", "HighMassRatio2", HighMa
 class HighMassRatio3 : public Sample
 {
 public:
-	explicit HighMassRatio3( Settings& settings )
-		: Sample( settings )
+	explicit HighMassRatio3( SampleContext* context )
+		: Sample( context )
 	{
-		if ( settings.restart == false )
+		if ( m_context->restart == false )
 		{
-			g_camera.m_center = { 0.0f, 16.5f };
-			g_camera.m_zoom = 25.0f;
+			m_context->camera.m_center = { 0.0f, 16.5f };
+			m_context->camera.m_zoom = 25.0f;
 		}
 
 		{
@@ -184,9 +183,9 @@ public:
 		}
 	}
 
-	static Sample* Create( Settings& settings )
+	static Sample* Create( SampleContext* context )
 	{
-		return new HighMassRatio3( settings );
+		return new HighMassRatio3( context );
 	}
 };
 
@@ -195,13 +194,13 @@ static int sampleIndex3 = RegisterSample( "Robustness", "HighMassRatio3", HighMa
 class OverlapRecovery : public Sample
 {
 public:
-	explicit OverlapRecovery( Settings& settings )
-		: Sample( settings )
+	explicit OverlapRecovery( SampleContext* context )
+		: Sample( context )
 	{
-		if ( settings.restart == false )
+		if ( m_context->restart == false )
 		{
-			g_camera.m_center = { 0.0f, 2.5f };
-			g_camera.m_zoom = 25.0f * 0.15f;
+			m_context->camera.m_center = { 0.0f, 2.5f };
+			m_context->camera.m_zoom = 3.75f;
 		}
 
 		m_bodyIds = nullptr;
@@ -209,7 +208,7 @@ public:
 		m_baseCount = 4;
 		m_overlap = 0.25f;
 		m_extent = 0.5f;
-		m_pushout = 3.0f;
+		m_pushOut = 3.0f;
 		m_hertz = 30.0f;
 		m_dampingRatio = 10.0f;
 
@@ -238,7 +237,7 @@ public:
 			b2DestroyBody( m_bodyIds[i] );
 		}
 
-		b2World_SetContactTuning( m_worldId, m_hertz, m_dampingRatio, m_pushout );
+		b2World_SetContactTuning( m_worldId, m_hertz, m_dampingRatio, m_pushOut );
 
 		b2BodyDef bodyDef = b2DefaultBodyDef();
 		bodyDef.type = b2_dynamicBody;
@@ -277,7 +276,7 @@ public:
 	void UpdateGui() override
 	{
 		float height = 210.0f;
-		ImGui::SetNextWindowPos( ImVec2( 10.0f, g_camera.m_height - height - 50.0f ), ImGuiCond_Once );
+		ImGui::SetNextWindowPos( ImVec2( 10.0f, m_context->camera.m_height - height - 50.0f ), ImGuiCond_Once );
 		ImGui::SetNextWindowSize( ImVec2( 220.0f, height ) );
 
 		ImGui::Begin( "Overlap Recovery", nullptr, ImGuiWindowFlags_NoResize );
@@ -287,8 +286,8 @@ public:
 		changed = changed || ImGui::SliderFloat( "Extent", &m_extent, 0.1f, 1.0f, "%.1f" );
 		changed = changed || ImGui::SliderInt( "Base Count", &m_baseCount, 1, 10 );
 		changed = changed || ImGui::SliderFloat( "Overlap", &m_overlap, 0.0f, 1.0f, "%.2f" );
-		changed = changed || ImGui::SliderFloat( "Pushout", &m_pushout, 0.0f, 10.0f, "%.1f" );
-		changed = changed || ImGui::SliderFloat( "Hertz", &m_hertz, 0.0f, 120.0f, "%.f" );
+		changed = changed || ImGui::SliderFloat( "Speed", &m_pushOut, 0.0f, 10.0f, "%.1f" );
+		changed = changed || ImGui::SliderFloat( "Hertz", &m_hertz, 0.0f, 240.0f, "%.f" );
 		changed = changed || ImGui::SliderFloat( "Damping Ratio", &m_dampingRatio, 0.0f, 20.0f, "%.1f" );
 		changed = changed || ImGui::Button( "Reset Scene" );
 
@@ -301,9 +300,9 @@ public:
 		ImGui::End();
 	}
 
-	static Sample* Create( Settings& settings )
+	static Sample* Create( SampleContext* context )
 	{
-		return new OverlapRecovery( settings );
+		return new OverlapRecovery( context );
 	}
 
 	b2BodyId* m_bodyIds;
@@ -311,7 +310,7 @@ public:
 	int32_t m_baseCount;
 	float m_overlap;
 	float m_extent;
-	float m_pushout;
+	float m_pushOut;
 	float m_hertz;
 	float m_dampingRatio;
 };
@@ -321,13 +320,13 @@ static int sampleIndex4 = RegisterSample( "Robustness", "Overlap Recovery", Over
 class TinyPyramid : public Sample
 {
 public:
-	explicit TinyPyramid( Settings& settings )
-		: Sample( settings )
+	explicit TinyPyramid( SampleContext* context )
+		: Sample( context )
 	{
-		if ( settings.restart == false )
+		if ( m_context->restart == false )
 		{
-			g_camera.m_center = { 0.0f, 0.8f };
-			g_camera.m_zoom = 1.0f;
+			m_context->camera.m_center = { 0.0f, 0.8f };
+			m_context->camera.m_zoom = 1.0f;
 		}
 
 		{
@@ -365,18 +364,183 @@ public:
 		}
 	}
 
-	void Step( Settings& settings ) override
+	void Step() override
 	{
 		DrawTextLine( "%.1fcm squares", 200.0f * m_extent );
-		Sample::Step( settings );
+		Sample::Step();
 	}
 
-	static Sample* Create( Settings& settings )
+	static Sample* Create( SampleContext* context )
 	{
-		return new TinyPyramid( settings );
+		return new TinyPyramid( context );
 	}
 
 	float m_extent;
 };
 
 static int sampleTinyPyramid = RegisterSample( "Robustness", "Tiny Pyramid", TinyPyramid::Create );
+
+// High gravity and high mass ratio
+class Cart : public Sample
+{
+public:
+	explicit Cart( SampleContext* context )
+		: Sample( context )
+	{
+		if ( m_context->restart == false )
+		{
+			m_context->camera.m_center = { 0.0f, 1.0f };
+			m_context->camera.m_zoom = 1.5f;
+		}
+
+		b2BodyId groundId;
+		{
+			b2BodyDef bodyDef = b2DefaultBodyDef();
+			bodyDef.position = { 0.0f, -1.0f };
+			groundId = b2CreateBody( m_worldId, &bodyDef );
+
+			b2ShapeDef shapeDef = b2DefaultShapeDef();
+			b2Polygon groundBox = b2MakeBox( 20.0f, 1.0f );
+			b2CreatePolygonShape( groundId, &shapeDef, &groundBox );
+		}
+
+		b2World_SetGravity( m_worldId, { 0, -22 } );
+
+		m_contactHertz = 30.0f;
+		m_contactDampingRatio = 10.0f;
+		m_contactSpeed = 3.0f;
+		b2World_SetContactTuning( m_worldId, m_contactHertz, m_contactDampingRatio, m_contactSpeed );
+
+		m_jointHertz = 60.0f;
+		m_jointDampingRatio = 1.0f;
+
+		m_chassisId = {};
+		m_wheelId1 = {};
+		m_wheelId2 = {};
+
+		CreateScene();
+	}
+
+	void CreateScene()
+	{
+		if (B2_IS_NON_NULL(m_chassisId))
+		{
+			b2DestroyBody( m_chassisId );
+		}
+
+		if (B2_IS_NON_NULL(m_wheelId1))
+		{
+			b2DestroyBody( m_wheelId1 );
+		}
+
+		if (B2_IS_NON_NULL(m_wheelId2))
+		{
+			b2DestroyBody( m_wheelId2 );
+		}
+
+		float yBase = 2.0f;
+
+		b2BodyDef bodyDef = b2DefaultBodyDef();
+		bodyDef.type = b2_dynamicBody;
+		bodyDef.position = { 0.0f, yBase };
+		m_chassisId = b2CreateBody( m_worldId, &bodyDef );
+
+		b2ShapeDef shapeDef = b2DefaultShapeDef();
+		shapeDef.density = 100.0f;
+
+		b2Polygon box = b2MakeOffsetBox( 0.5f, 0.25f, { 0.0f, 0.25f }, b2Rot_identity );
+		b2CreatePolygonShape( m_chassisId, &shapeDef, &box );
+
+		shapeDef = b2DefaultShapeDef();
+		shapeDef.material.rollingResistance = 0.02f;
+		shapeDef.density = 10.0f;
+
+		b2Circle circle = { b2Vec2_zero, 0.1f };
+		bodyDef.position = { -0.4f, yBase - 0.15f };
+		m_wheelId1 = b2CreateBody( m_worldId, &bodyDef );
+		b2CreateCircleShape( m_wheelId1, &shapeDef, &circle );
+
+		bodyDef.position = { 0.4f, yBase - 0.15f };
+		m_wheelId2 = b2CreateBody( m_worldId, &bodyDef );
+		b2CreateCircleShape( m_wheelId2, &shapeDef, &circle );
+
+		b2RevoluteJointDef jointDef = b2DefaultRevoluteJointDef();
+		jointDef.bodyIdA = m_chassisId;
+		jointDef.bodyIdB = m_wheelId1;
+		jointDef.localAnchorA = { -0.4f, -0.15f };
+		jointDef.localAnchorB = { 0.0f, 0.0f };
+
+		m_jointId1 = b2CreateRevoluteJoint( m_worldId, &jointDef );
+		b2Joint_SetConstraintTuning( m_jointId1, m_jointHertz, m_jointDampingRatio );
+
+		jointDef.bodyIdA = m_chassisId;
+		jointDef.bodyIdB = m_wheelId2;
+		jointDef.localAnchorA = { 0.4f, -0.15f };
+		jointDef.localAnchorB = { 0.0f, 0.0f };
+
+		m_jointId2 = b2CreateRevoluteJoint( m_worldId, &jointDef );
+		b2Joint_SetConstraintTuning( m_jointId2, m_jointHertz, m_jointDampingRatio );
+	}
+
+	void UpdateGui() override
+	{
+		float height = 240.0f;
+		ImGui::SetNextWindowPos( ImVec2( 10.0f, m_context->camera.m_height - height - 50.0f ), ImGuiCond_Once );
+		ImGui::SetNextWindowSize( ImVec2( 320.0f, height ) );
+
+		ImGui::Begin( "Cart", nullptr, ImGuiWindowFlags_NoResize );
+		ImGui::PushItemWidth( 200.0f );
+
+		bool changed = false;
+		ImGui::Text( "Contact" );
+		changed = changed || ImGui::SliderFloat( "Hertz##contact", &m_contactHertz, 0.0f, 240.0f, "%.f" );
+		changed = changed || ImGui::SliderFloat( "Damping Ratio##contact", &m_contactDampingRatio, 0.0f, 1000.0f, "%.f" );
+		changed = changed || ImGui::SliderFloat( "Speed", &m_contactSpeed, 0.0f, 5.0f, "%.1f" );
+
+		if ( changed )
+		{
+			b2World_SetContactTuning( m_worldId, m_contactHertz, m_contactDampingRatio, m_contactSpeed );
+			CreateScene();
+		}
+
+		ImGui::Separator();
+
+		changed = false;
+		ImGui::Text( "Joint" );
+		changed = changed || ImGui::SliderFloat( "Hertz##joint", &m_jointHertz, 0.0f, 240.0f, "%.f" );
+		changed = changed || ImGui::SliderFloat( "Damping Ratio##joint", &m_jointDampingRatio, 0.0f, 1000.0f, "%.f" );
+
+		ImGui::Separator();
+
+		changed = changed || ImGui::Button( "Reset Scene" );
+
+		if ( changed )
+		{
+			b2Joint_SetConstraintTuning( m_jointId1, m_jointHertz, m_jointDampingRatio );
+			b2Joint_SetConstraintTuning( m_jointId2, m_jointHertz, m_jointDampingRatio );
+			CreateScene();
+		}
+
+		ImGui::PopItemWidth();
+		ImGui::End();
+	}
+
+	static Sample* Create( SampleContext* context )
+	{
+		return new Cart( context );
+	}
+
+	b2BodyId m_chassisId;
+	b2BodyId m_wheelId1;
+	b2BodyId m_wheelId2;
+	b2JointId m_jointId1;
+	b2JointId m_jointId2;
+
+	float m_contactHertz;
+	float m_contactDampingRatio;
+	float m_contactSpeed;
+	float m_jointHertz;
+	float m_jointDampingRatio;
+};
+
+static int sampleCart = RegisterSample( "Robustness", "Cart", Cart::Create );
