@@ -8,6 +8,6 @@
 uint32_t CreateProgramFromFiles( const char* vertexPath, const char* fragmentPath );
 uint32_t CreateProgramFromStrings( const char* vertexString, const char* fragmentString );
 
-void CheckErrorGL();
+void CheckOpenGL();
 void DumpInfoGL();
 void PrintLogGL( int object );
