@@ -34,7 +34,6 @@ void b2CreateBroadPhase( b2BroadPhase* bp )
 	//	fprintf(s_file, "============\n\n");
 	// }
 
-	bp->proxyCount = 0;
 	bp->moveSet = b2CreateSet( 16 );
 	bp->moveArray = b2IntArray_Create( 16 );
 	bp->moveResults = NULL;
@@ -107,8 +106,6 @@ void b2BroadPhase_DestroyProxy( b2BroadPhase* bp, int proxyKey )
 	B2_ASSERT( bp->moveArray.count == (int)bp->moveSet.count );
 	b2UnBufferMove( bp, proxyKey );
 
-	--bp->proxyCount;
-
 	b2BodyType proxyType = B2_PROXY_TYPE( proxyKey );
 	int proxyId = B2_PROXY_ID( proxyKey );
 
@@ -187,7 +184,7 @@ static bool b2PairQueryCallback( int proxyId, uint64_t userData, void* context )
 	// I had an optimization here to skip checking the move set if this is a query into
 	// the static tree. The assumption is that the static proxies are never in the move set
 	// so there is no risk of duplication. However, this is not true with
-	// b2ShapeDef::forceContactCreation, b2ShapeDef::isSensor, or when a static shape is modified.
+	// b2ShapeDef::invokeContactCreation or when a static shape is modified.
 	// There can easily be scenarios where the static proxy is in the moveSet but the dynamic proxy is not.
 	// I could have some flag to indicate that there are any static bodies in the moveSet.
 	
