@@ -3,7 +3,6 @@
 
 #include "island.h"
 
-#include "array.h"
 #include "body.h"
 #include "contact.h"
 #include "core.h"
@@ -13,28 +12,30 @@
 
 #include <stddef.h>
 
+B2_ARRAY_SOURCE( b2Island, b2Island )
+B2_ARRAY_SOURCE( b2IslandSim, b2IslandSim )
+
 b2Island* b2CreateIsland( b2World* world, int setIndex )
 {
 	B2_ASSERT( setIndex == b2_awakeSet || setIndex >= b2_firstSleepingSet );
 
 	int islandId = b2AllocId( &world->islandIdPool );
 
-	if ( islandId == b2Array( world->islandArray ).count )
+	if ( islandId == world->islands.count )
 	{
 		b2Island emptyIsland = { 0 };
-		b2Array_Push( world->islandArray, emptyIsland );
+		b2IslandArray_Push( &world->islands, emptyIsland );
 	}
 	else
 	{
-		B2_ASSERT( world->islandArray[islandId].setIndex == B2_NULL_INDEX );
+		B2_ASSERT( world->islands.data[islandId].setIndex == B2_NULL_INDEX );
 	}
 
-	b2CheckIndex( world->solverSetArray, setIndex );
-	b2SolverSet* set = world->solverSetArray + setIndex;
+	b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, setIndex );
 
-	b2Island* island = world->islandArray + islandId;
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 	island->setIndex = setIndex;
-	island->localIndex = set->islands.count;
+	island->localIndex = set->islandSims.count;
 	island->islandId = islandId;
 	island->headBody = B2_NULL_INDEX;
 	island->tailBody = B2_NULL_INDEX;
@@ -48,7 +49,7 @@ b2Island* b2CreateIsland( b2World* world, int setIndex )
 	island->parentIsland = B2_NULL_INDEX;
 	island->constraintRemoveCount = 0;
 
-	b2IslandSim* islandSim = b2AddIsland( &set->islands );
+	b2IslandSim* islandSim = b2IslandSimArray_Add( &set->islandSims );
 	islandSim->islandId = islandId;
 
 	return island;
@@ -56,18 +57,21 @@ b2Island* b2CreateIsland( b2World* world, int setIndex )
 
 void b2DestroyIsland( b2World* world, int islandId )
 {
+	if (world->splitIslandId == islandId)
+	{
+		world->splitIslandId = B2_NULL_INDEX;
+	}
+
 	// assume island is empty
-	b2CheckIndex( world->islandArray, islandId );
-	b2Island* island = world->islandArray + islandId;
-	b2CheckIndex( world->solverSetArray, island->setIndex );
-	b2SolverSet* set = world->solverSetArray + island->setIndex;
-	int movedIndex = b2RemoveIsland( &set->islands, island->localIndex );
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
+	b2SolverSet* set = b2SolverSetArray_Get( &world->solverSets, island->setIndex );
+	int movedIndex = b2IslandSimArray_RemoveSwap( &set->islandSims, island->localIndex );
 	if ( movedIndex != B2_NULL_INDEX )
 	{
 		// Fix index on moved element
-		b2IslandSim* movedElement = set->islands.data + island->localIndex;
+		b2IslandSim* movedElement = set->islandSims.data + island->localIndex;
 		int movedId = movedElement->islandId;
-		b2Island* movedIsland = world->islandArray + movedId;
+		b2Island* movedIsland = b2IslandArray_Get( &world->islands, movedId );
 		B2_ASSERT( movedIsland->localIndex == movedIndex );
 		movedIsland->localIndex = island->localIndex;
 	}
@@ -79,26 +83,18 @@ void b2DestroyIsland( b2World* world, int islandId )
 	b2FreeId( &world->islandIdPool, islandId );
 }
 
-b2Island* b2GetIsland( b2World* world, int islandId )
-{
-	b2CheckIndex( world->islandArray, islandId );
-	return world->islandArray + islandId;
-}
-
 static void b2AddContactToIsland( b2World* world, int islandId, b2Contact* contact )
 {
 	B2_ASSERT( contact->islandId == B2_NULL_INDEX );
 	B2_ASSERT( contact->islandPrev == B2_NULL_INDEX );
 	B2_ASSERT( contact->islandNext == B2_NULL_INDEX );
 
-	b2CheckIndex( world->islandArray, islandId );
-	b2Island* island = world->islandArray + islandId;
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 
 	if ( island->headContact != B2_NULL_INDEX )
 	{
 		contact->islandNext = island->headContact;
-		b2CheckIndex( world->contactArray, island->headContact );
-		b2Contact* headContact = world->contactArray + island->headContact;
+		b2Contact* headContact = b2ContactArray_Get( &world->contacts, island->headContact);
 		headContact->islandPrev = contact->contactId;
 	}
 
@@ -119,13 +115,13 @@ static void b2AddContactToIsland( b2World* world, int islandId, b2Contact* conta
 // https://en.wikipedia.org/wiki/Disjoint-set_data_structure
 void b2LinkContact( b2World* world, b2Contact* contact )
 {
-	B2_ASSERT( ( contact->flags & b2_contactTouchingFlag ) != 0 && ( contact->flags & b2_contactSensorFlag ) == 0 );
+	B2_ASSERT( ( contact->flags & b2_contactTouchingFlag ) != 0 );
 
 	int bodyIdA = contact->edges[0].bodyId;
 	int bodyIdB = contact->edges[1].bodyId;
 
-	b2Body* bodyA = b2GetBody( world, bodyIdA );
-	b2Body* bodyB = b2GetBody( world, bodyIdB );
+	b2Body* bodyA = b2BodyArray_Get( &world->bodies, bodyIdA );
+	b2Body* bodyB = b2BodyArray_Get( &world->bodies, bodyIdB );
 
 	B2_ASSERT( bodyA->setIndex != b2_disabledSet && bodyB->setIndex != b2_disabledSet );
 	B2_ASSERT( bodyA->setIndex != b2_staticSet || bodyB->setIndex != b2_staticSet );
@@ -161,11 +157,11 @@ void b2LinkContact( b2World* world, b2Contact* contact )
 	b2Island* islandA = NULL;
 	if ( islandIdA != B2_NULL_INDEX )
 	{
-		islandA = b2GetIsland( world, islandIdA );
+		islandA = b2IslandArray_Get( &world->islands, islandIdA );
 		int parentId = islandA->parentIsland;
 		while ( parentId != B2_NULL_INDEX )
 		{
-			b2Island* parent = b2GetIsland( world, parentId );
+			b2Island* parent = b2IslandArray_Get( &world->islands, parentId );
 			if ( parent->parentIsland != B2_NULL_INDEX )
 			{
 				// path compression
@@ -182,11 +178,11 @@ void b2LinkContact( b2World* world, b2Contact* contact )
 	b2Island* islandB = NULL;
 	if ( islandIdB != B2_NULL_INDEX )
 	{
-		islandB = b2GetIsland( world, islandIdB );
+		islandB = b2IslandArray_Get( &world->islands, islandIdB );
 		int parentId = islandB->parentIsland;
 		while ( islandB->parentIsland != B2_NULL_INDEX )
 		{
-			b2Island* parent = b2GetIsland( world, parentId );
+			b2Island* parent = b2IslandArray_Get( &world->islands, parentId );
 			if ( parent->parentIsland != B2_NULL_INDEX )
 			{
 				// path compression
@@ -217,31 +213,29 @@ void b2LinkContact( b2World* world, b2Contact* contact )
 	{
 		b2AddContactToIsland( world, islandIdB, contact );
 	}
+
+	// todo why not merge the islands right here?
 }
 
 // This is called when a contact no longer has contact points or when a contact is destroyed.
 void b2UnlinkContact( b2World* world, b2Contact* contact )
 {
-	B2_ASSERT( ( contact->flags & b2_contactSensorFlag ) == 0 );
 	B2_ASSERT( contact->islandId != B2_NULL_INDEX );
 
 	// remove from island
 	int islandId = contact->islandId;
-	b2CheckIndex( world->islandArray, islandId );
-	b2Island* island = b2GetIsland( world, islandId );
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 
 	if ( contact->islandPrev != B2_NULL_INDEX )
 	{
-		b2CheckIndex( world->contactArray, contact->islandPrev );
-		b2Contact* prevContact = world->contactArray + contact->islandPrev;
+		b2Contact* prevContact = b2ContactArray_Get( &world->contacts, contact->islandPrev);
 		B2_ASSERT( prevContact->islandNext == contact->contactId );
 		prevContact->islandNext = contact->islandNext;
 	}
 
 	if ( contact->islandNext != B2_NULL_INDEX )
 	{
-		b2CheckIndex( world->contactArray, contact->islandNext );
-		b2Contact* nextContact = world->contactArray + contact->islandNext;
+		b2Contact* nextContact = b2ContactArray_Get( &world->contacts, contact->islandNext );
 		B2_ASSERT( nextContact->islandPrev == contact->contactId );
 		nextContact->islandPrev = contact->islandPrev;
 	}
@@ -273,13 +267,12 @@ static void b2AddJointToIsland( b2World* world, int islandId, b2Joint* joint )
 	B2_ASSERT( joint->islandPrev == B2_NULL_INDEX );
 	B2_ASSERT( joint->islandNext == B2_NULL_INDEX );
 
-	b2CheckIndex( world->islandArray, islandId );
-	b2Island* island = world->islandArray + islandId;
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 
 	if ( island->headJoint != B2_NULL_INDEX )
 	{
 		joint->islandNext = island->headJoint;
-		b2Joint* headJoint = b2GetJoint( world, island->headJoint );
+		b2Joint* headJoint = b2JointArray_Get( &world->joints, island->headJoint );
 		headJoint->islandPrev = joint->jointId;
 	}
 
@@ -295,10 +288,10 @@ static void b2AddJointToIsland( b2World* world, int islandId, b2Joint* joint )
 	b2ValidateIsland( world, islandId );
 }
 
-void b2LinkJoint( b2World* world, b2Joint* joint )
+void b2LinkJoint( b2World* world, b2Joint* joint, bool mergeIslands )
 {
-	b2Body* bodyA = b2GetBody( world, joint->edges[0].bodyId );
-	b2Body* bodyB = b2GetBody( world, joint->edges[1].bodyId );
+	b2Body* bodyA = b2BodyArray_Get( &world->bodies, joint->edges[0].bodyId );
+	b2Body* bodyB = b2BodyArray_Get( &world->bodies, joint->edges[1].bodyId );
 
 	if ( bodyA->setIndex == b2_awakeSet && bodyB->setIndex >= b2_firstSleepingSet )
 	{
@@ -325,10 +318,10 @@ void b2LinkJoint( b2World* world, b2Joint* joint )
 	b2Island* islandA = NULL;
 	if ( islandIdA != B2_NULL_INDEX )
 	{
-		islandA = b2GetIsland( world, islandIdA );
+		islandA = b2IslandArray_Get( &world->islands, islandIdA );
 		while ( islandA->parentIsland != B2_NULL_INDEX )
 		{
-			b2Island* parent = b2GetIsland( world, islandA->parentIsland );
+			b2Island* parent = b2IslandArray_Get( &world->islands, islandA->parentIsland );
 			if ( parent->parentIsland != B2_NULL_INDEX )
 			{
 				// path compression
@@ -344,10 +337,10 @@ void b2LinkJoint( b2World* world, b2Joint* joint )
 	b2Island* islandB = NULL;
 	if ( islandIdB != B2_NULL_INDEX )
 	{
-		islandB = b2GetIsland( world, islandIdB );
+		islandB = b2IslandArray_Get( &world->islands, islandIdB );
 		while ( islandB->parentIsland != B2_NULL_INDEX )
 		{
-			b2Island* parent = b2GetIsland( world, islandB->parentIsland );
+			b2Island* parent = b2IslandArray_Get( &world->islands, islandB->parentIsland );
 			if ( parent->parentIsland != B2_NULL_INDEX )
 			{
 				// path compression
@@ -377,6 +370,15 @@ void b2LinkJoint( b2World* world, b2Joint* joint )
 	{
 		b2AddJointToIsland( world, islandIdB, joint );
 	}
+
+	// Joints need to have islands merged immediately when they are created
+	// to keep the island graph valid.
+	// However, when a body type is being changed the merge can be deferred until
+	// all joints are linked.
+	if (mergeIslands)
+	{
+		b2MergeAwakeIslands( world );
+	}
 }
 
 void b2UnlinkJoint( b2World* world, b2Joint* joint )
@@ -385,19 +387,18 @@ void b2UnlinkJoint( b2World* world, b2Joint* joint )
 
 	// remove from island
 	int islandId = joint->islandId;
-	b2CheckIndex( world->islandArray, islandId );
-	b2Island* island = world->islandArray + islandId;
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 
 	if ( joint->islandPrev != B2_NULL_INDEX )
 	{
-		b2Joint* prevJoint = b2GetJoint( world, joint->islandPrev );
+		b2Joint* prevJoint = b2JointArray_Get( &world->joints, joint->islandPrev );
 		B2_ASSERT( prevJoint->islandNext == joint->jointId );
 		prevJoint->islandNext = joint->islandNext;
 	}
 
 	if ( joint->islandNext != B2_NULL_INDEX )
 	{
-		b2Joint* nextJoint = b2GetJoint( world, joint->islandNext );
+		b2Joint* nextJoint = b2JointArray_Get( &world->joints, joint->islandNext );
 		B2_ASSERT( nextJoint->islandPrev == joint->jointId );
 		nextJoint->islandPrev = joint->islandPrev;
 	}
@@ -430,15 +431,14 @@ static void b2MergeIsland( b2World* world, b2Island* island )
 	B2_ASSERT( island->parentIsland != B2_NULL_INDEX );
 
 	int rootId = island->parentIsland;
-	b2CheckIndex( world->islandArray, rootId );
-	b2Island* rootIsland = world->islandArray + rootId;
+	b2Island* rootIsland = b2IslandArray_Get( &world->islands, rootId );
 	B2_ASSERT( rootIsland->parentIsland == B2_NULL_INDEX );
 
 	// remap island indices
 	int bodyId = island->headBody;
 	while ( bodyId != B2_NULL_INDEX )
 	{
-		b2Body* body = b2GetBody( world, bodyId );
+		b2Body* body = b2BodyArray_Get( &world->bodies, bodyId );
 		body->islandId = rootId;
 		bodyId = body->islandNext;
 	}
@@ -446,8 +446,7 @@ static void b2MergeIsland( b2World* world, b2Island* island )
 	int contactId = island->headContact;
 	while ( contactId != B2_NULL_INDEX )
 	{
-		b2CheckIndex( world->contactArray, contactId );
-		b2Contact* contact = world->contactArray + contactId;
+		b2Contact* contact = b2ContactArray_Get( &world->contacts, contactId );
 		contact->islandId = rootId;
 		contactId = contact->islandNext;
 	}
@@ -455,19 +454,19 @@ static void b2MergeIsland( b2World* world, b2Island* island )
 	int jointId = island->headJoint;
 	while ( jointId != B2_NULL_INDEX )
 	{
-		b2Joint* joint = b2GetJoint( world, jointId );
+		b2Joint* joint = b2JointArray_Get( &world->joints, jointId );
 		joint->islandId = rootId;
 		jointId = joint->islandNext;
 	}
 
 	// connect body lists
 	B2_ASSERT( rootIsland->tailBody != B2_NULL_INDEX );
-	b2Body* tailBody = b2GetBody( world, rootIsland->tailBody );
+	b2Body* tailBody = b2BodyArray_Get( &world->bodies, rootIsland->tailBody );
 	B2_ASSERT( tailBody->islandNext == B2_NULL_INDEX );
 	tailBody->islandNext = island->headBody;
 
 	B2_ASSERT( island->headBody != B2_NULL_INDEX );
-	b2Body* headBody = b2GetBody( world, island->headBody );
+	b2Body* headBody = b2BodyArray_Get( &world->bodies, island->headBody );
 	B2_ASSERT( headBody->islandPrev == B2_NULL_INDEX );
 	headBody->islandPrev = rootIsland->tailBody;
 
@@ -489,13 +488,11 @@ static void b2MergeIsland( b2World* world, b2Island* island )
 		B2_ASSERT( island->tailContact != B2_NULL_INDEX && island->contactCount > 0 );
 		B2_ASSERT( rootIsland->tailContact != B2_NULL_INDEX && rootIsland->contactCount > 0 );
 
-		b2CheckIndex( world->contactArray, rootIsland->tailContact );
-		b2Contact* tailContact = world->contactArray + rootIsland->tailContact;
+		b2Contact* tailContact = b2ContactArray_Get( &world->contacts, rootIsland->tailContact );
 		B2_ASSERT( tailContact->islandNext == B2_NULL_INDEX );
 		tailContact->islandNext = island->headContact;
 
-		b2CheckIndex( world->contactArray, island->headContact );
-		b2Contact* headContact = world->contactArray + island->headContact;
+		b2Contact* headContact = b2ContactArray_Get( &world->contacts, island->headContact );
 		B2_ASSERT( headContact->islandPrev == B2_NULL_INDEX );
 		headContact->islandPrev = rootIsland->tailContact;
 
@@ -517,11 +514,11 @@ static void b2MergeIsland( b2World* world, b2Island* island )
 		B2_ASSERT( island->tailJoint != B2_NULL_INDEX && island->jointCount > 0 );
 		B2_ASSERT( rootIsland->tailJoint != B2_NULL_INDEX && rootIsland->jointCount > 0 );
 
-		b2Joint* tailJoint = b2GetJoint( world, rootIsland->tailJoint );
+		b2Joint* tailJoint = b2JointArray_Get( &world->joints, rootIsland->tailJoint );
 		B2_ASSERT( tailJoint->islandNext == B2_NULL_INDEX );
 		tailJoint->islandNext = island->headJoint;
 
-		b2Joint* headJoint = b2GetJoint( world, island->headJoint );
+		b2Joint* headJoint = b2JointArray_Get( &world->joints, island->headJoint );
 		B2_ASSERT( headJoint->islandPrev == B2_NULL_INDEX );
 		headJoint->islandPrev = rootIsland->tailJoint;
 
@@ -543,10 +540,9 @@ void b2MergeAwakeIslands( b2World* world )
 {
 	b2TracyCZoneNC( merge_islands, "Merge Islands", b2_colorMediumTurquoise, true );
 
-	b2SolverSet* awakeSet = world->solverSetArray + b2_awakeSet;
-	b2IslandSim* islandSims = awakeSet->islands.data;
-	int awakeIslandCount = awakeSet->islands.count;
-	b2Island* islands = world->islandArray;
+	b2SolverSet* awakeSet = b2SolverSetArray_Get( &world->solverSets, b2_awakeSet );
+	b2IslandSim* islandSims = awakeSet->islandSims.data;
+	int awakeIslandCount = awakeSet->islandSims.count;
 
 	// Step 1: Ensure every child island points to its root island. This avoids merging a child island with
 	// a parent island that has already been merged with a grand-parent island.
@@ -554,16 +550,14 @@ void b2MergeAwakeIslands( b2World* world )
 	{
 		int islandId = islandSims[i].islandId;
 
-		b2CheckIndex( islands, islandId );
-		b2Island* island = islands + islandId;
+		b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 
 		// find the root island
 		int rootId = islandId;
 		b2Island* rootIsland = island;
 		while ( rootIsland->parentIsland != B2_NULL_INDEX )
 		{
-			b2CheckIndex( islands, rootIsland->parentIsland );
-			b2Island* parent = islands + rootIsland->parentIsland;
+			b2Island* parent = b2IslandArray_Get( &world->islands, rootIsland->parentIsland );
 			if ( parent->parentIsland != B2_NULL_INDEX )
 			{
 				// path compression
@@ -585,8 +579,7 @@ void b2MergeAwakeIslands( b2World* world )
 	for ( int i = awakeIslandCount - 1; i >= 0; --i )
 	{
 		int islandId = islandSims[i].islandId;
-		b2CheckIndex( islands, islandId );
-		b2Island* island = islands + islandId;
+		b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 
 		if ( island->parentIsland == B2_NULL_INDEX )
 		{
@@ -608,8 +601,7 @@ void b2MergeAwakeIslands( b2World* world )
 
 void b2SplitIsland( b2World* world, int baseId )
 {
-	b2CheckIndex( world->islandArray, baseId );
-	b2Island* baseIsland = world->islandArray + baseId;
+	b2Island* baseIsland = b2IslandArray_Get( &world->islands, baseId );
 	int setIndex = baseIsland->setIndex;
 
 	if ( setIndex != b2_awakeSet )
@@ -628,14 +620,12 @@ void b2SplitIsland( b2World* world, int baseId )
 
 	int bodyCount = baseIsland->bodyCount;
 
-	b2Body* bodies = world->bodyArray;
-	b2Contact* contacts = world->contactArray;
-
-	b2StackAllocator* alloc = &world->stackAllocator;
+	b2Body* bodies = world->bodies.data;
+	b2ArenaAllocator* alloc = &world->arena;
 
 	// No lock is needed because I ensure the allocator is not used while this task is active.
-	int* stack = b2AllocateStackItem( alloc, bodyCount * sizeof( int ), "island stack" );
-	int* bodyIds = b2AllocateStackItem( alloc, bodyCount * sizeof( int ), "body ids" );
+	int* stack = b2AllocateArenaItem( alloc, bodyCount * sizeof( int ), "island stack" );
+	int* bodyIds = b2AllocateArenaItem( alloc, bodyCount * sizeof( int ), "body ids" );
 
 	// Build array containing all body indices from base island. These
 	// serve as seed bodies for the depth first search (DFS).
@@ -658,7 +648,7 @@ void b2SplitIsland( b2World* world, int baseId )
 	int nextContactId = baseIsland->headContact;
 	while ( nextContactId != B2_NULL_INDEX )
 	{
-		b2Contact* contact = contacts + nextContactId;
+		b2Contact* contact = b2ContactArray_Get( &world->contacts, nextContactId );
 		contact->isMarked = false;
 		nextContactId = contact->islandNext;
 	}
@@ -667,7 +657,7 @@ void b2SplitIsland( b2World* world, int baseId )
 	int nextJoint = baseIsland->headJoint;
 	while ( nextJoint != B2_NULL_INDEX )
 	{
-		b2Joint* joint = b2GetJoint( world, nextJoint );
+		b2Joint* joint = b2JointArray_Get( &world->joints, nextJoint );
 		joint->isMarked = false;
 		nextJoint = joint->islandNext;
 	}
@@ -732,8 +722,7 @@ void b2SplitIsland( b2World* world, int baseId )
 				int contactId = contactKey >> 1;
 				int edgeIndex = contactKey & 1;
 
-				b2CheckIndex( world->contactArray, contactId );
-				b2Contact* contact = world->contactArray + contactId;
+				b2Contact* contact = b2ContactArray_Get( &world->contacts, contactId );
 				B2_ASSERT( contact->contactId == contactId );
 
 				// Next key
@@ -745,12 +734,6 @@ void b2SplitIsland( b2World* world, int baseId )
 					continue;
 				}
 
-				// Skip sensors
-				if ( contact->flags & b2_contactSensorFlag )
-				{
-					continue;
-				}
-
 				// Is this contact enabled and touching?
 				if ( ( contact->flags & b2_contactTouchingFlag ) == 0 )
 				{
@@ -775,8 +758,7 @@ void b2SplitIsland( b2World* world, int baseId )
 				contact->islandId = islandId;
 				if ( island->tailContact != B2_NULL_INDEX )
 				{
-					b2CheckIndex( world->contactArray, island->tailContact );
-					b2Contact* tailContact = world->contactArray + island->tailContact;
+					b2Contact* tailContact = b2ContactArray_Get( &world->contacts, island->tailContact );
 					tailContact->islandNext = contactId;
 				}
 				contact->islandPrev = island->tailContact;
@@ -798,7 +780,7 @@ void b2SplitIsland( b2World* world, int baseId )
 				int jointId = jointKey >> 1;
 				int edgeIndex = jointKey & 1;
 
-				b2Joint* joint = b2GetJoint( world, jointId );
+				b2Joint* joint = b2JointArray_Get( &world->joints, jointId );
 				B2_ASSERT( joint->jointId == jointId );
 
 				// Next key
@@ -834,7 +816,7 @@ void b2SplitIsland( b2World* world, int baseId )
 				joint->islandId = islandId;
 				if ( island->tailJoint != B2_NULL_INDEX )
 				{
-					b2Joint* tailJoint = b2GetJoint( world, island->tailJoint );
+					b2Joint* tailJoint = b2JointArray_Get( &world->joints, island->tailJoint );
 					tailJoint->islandNext = jointId;
 				}
 				joint->islandPrev = island->tailJoint;
@@ -853,8 +835,8 @@ void b2SplitIsland( b2World* world, int baseId )
 		b2ValidateIsland( world, islandId );
 	}
 
-	b2FreeStackItem( alloc, bodyIds );
-	b2FreeStackItem( alloc, stack );
+	b2FreeArenaItem( alloc, bodyIds );
+	b2FreeArenaItem( alloc, stack );
 }
 
 // Split an island because some contacts and/or joints have been removed.
@@ -866,34 +848,30 @@ void b2SplitIsland( b2World* world, int baseId )
 // are interacting with these data structures.
 void b2SplitIslandTask( int startIndex, int endIndex, uint32_t threadIndex, void* context )
 {
-	b2TracyCZoneNC( split, "Split Island", b2_colorHoneydew, true );
+	b2TracyCZoneNC( split, "Split Island", b2_colorOlive, true );
 
-	B2_MAYBE_UNUSED( startIndex );
-	B2_MAYBE_UNUSED( endIndex );
-	B2_MAYBE_UNUSED( threadIndex );
+	B2_UNUSED( startIndex, endIndex, threadIndex );
 
-	b2Timer timer = b2CreateTimer();
+	uint64_t ticks = b2GetTicks();
 	b2World* world = context;
 
 	B2_ASSERT( world->splitIslandId != B2_NULL_INDEX );
 
 	b2SplitIsland( world, world->splitIslandId );
 
-	world->profile.splitIslands += b2GetMilliseconds( &timer );
+	world->profile.splitIslands += b2GetMilliseconds( ticks );
 	b2TracyCZoneEnd( split );
 }
 
 #if B2_VALIDATE
 void b2ValidateIsland( b2World* world, int islandId )
 {
-	b2CheckIndex( world->islandArray, islandId );
-	b2Island* island = world->islandArray + islandId;
+	b2Island* island = b2IslandArray_Get( &world->islands, islandId );
 	B2_ASSERT( island->islandId == islandId );
 	B2_ASSERT( island->setIndex != B2_NULL_INDEX );
 	B2_ASSERT( island->headBody != B2_NULL_INDEX );
 
 	{
-		b2Body* bodies = world->bodyArray;
 		B2_ASSERT( island->tailBody != B2_NULL_INDEX );
 		B2_ASSERT( island->bodyCount > 0 );
 		if ( island->bodyCount > 1 )
@@ -906,8 +884,7 @@ void b2ValidateIsland( b2World* world, int islandId )
 		int bodyId = island->headBody;
 		while ( bodyId != B2_NULL_INDEX )
 		{
-			b2CheckIndex( bodies, bodyId );
-			b2Body* body = bodies + bodyId;
+			b2Body* body = b2BodyArray_Get(&world->bodies, bodyId);
 			B2_ASSERT( body->islandId == islandId );
 			B2_ASSERT( body->setIndex == island->setIndex );
 			count += 1;
@@ -936,8 +913,7 @@ void b2ValidateIsland( b2World* world, int islandId )
 		int contactId = island->headContact;
 		while ( contactId != B2_NULL_INDEX )
 		{
-			b2CheckIndex( world->contactArray, contactId );
-			b2Contact* contact = world->contactArray + contactId;
+			b2Contact* contact = b2ContactArray_Get( &world->contacts, contactId );
 			B2_ASSERT( contact->setIndex == island->setIndex );
 			B2_ASSERT( contact->islandId == islandId );
 			count += 1;
@@ -971,8 +947,7 @@ void b2ValidateIsland( b2World* world, int islandId )
 		int jointId = island->headJoint;
 		while ( jointId != B2_NULL_INDEX )
 		{
-			b2CheckIndex( world->jointArray, jointId );
-			b2Joint* joint = world->jointArray + jointId;
+			b2Joint* joint = b2JointArray_Get( &world->joints, jointId );
 			B2_ASSERT( joint->setIndex == island->setIndex );
 			count += 1;
 
@@ -996,7 +971,7 @@ void b2ValidateIsland( b2World* world, int islandId )
 
 void b2ValidateIsland( b2World* world, int islandId )
 {
-	B2_MAYBE_UNUSED( world );
-	B2_MAYBE_UNUSED( islandId );
+	B2_UNUSED( world );
+	B2_UNUSED( islandId );
 }
 #endif
