@@ -22,6 +22,10 @@
 #include <stdbool.h>
 #include <stddef.h>
 
+// todo testing
+#define ITERATIONS 1
+#define RELAX_ITERATIONS 1
+
 // Compare to SDL_CPUPauseInstruction
 #if ( defined( __GNUC__ ) || defined( __clang__ ) ) && ( defined( __i386__ ) || defined( __x86_64__ ) )
 static inline void b2Pause( void )
@@ -360,7 +364,7 @@ static bool b2ContinuousQueryCallback( int proxyId, uint64_t userData, void* con
 		}
 	}
 
-	if ( didHit && ( shape->enablePreSolveEvents || fastShape->enablePreSolveEvents ) )
+	if ( didHit && ( shape->enablePreSolveEvents || fastShape->enablePreSolveEvents ) && world->preSolveFcn != NULL )
 	{
 		// Pre-solve is expensive because I need to compute a temporary manifold
 		b2Transform transformA = b2GetSweepTransform( &input.sweepA, hitFraction );
@@ -767,16 +771,13 @@ static void b2ExecuteBlock( b2SolverStage* stage, b2StepContext* context, b2Solv
 			break;
 
 		case b2_stageWarmStart:
-			if ( context->world->enableWarmStarting )
+			if ( blockType == b2_graphContactBlock )
 			{
-				if ( blockType == b2_graphContactBlock )
-				{
-					b2WarmStartContactsTask( startIndex, endIndex, context, stage->colorIndex );
-				}
-				else if ( blockType == b2_graphJointBlock )
-				{
-					b2WarmStartJointsTask( startIndex, endIndex, context, stage->colorIndex );
-				}
+				b2WarmStartContactsTask( startIndex, endIndex, context, stage->colorIndex );
+			}
+			else if ( blockType == b2_graphJointBlock )
+			{
+				b2WarmStartJointsTask( startIndex, endIndex, context, stage->colorIndex );
 			}
 			break;
 
@@ -1020,17 +1021,21 @@ static void b2SolverTask( int startIndex, int endIndex, uint32_t threadIndexIgno
 
 			// solve constraints
 			bool useBias = true;
-			b2SolveOverflowJoints( context, useBias );
-			b2SolveOverflowContacts( context, useBias );
 
-			for ( int colorIndex = 0; colorIndex < activeColorCount; ++colorIndex )
+			for ( int j = 0; j < ITERATIONS; ++j )
 			{
-				syncBits = ( graphSyncIndex << 16 ) | iterStageIndex;
-				B2_ASSERT( stages[iterStageIndex].type == b2_stageSolve );
-				b2ExecuteMainStage( stages + iterStageIndex, context, syncBits );
-				iterStageIndex += 1;
+				b2SolveOverflowJoints( context, useBias );
+				b2SolveOverflowContacts( context, useBias );
+
+				for ( int colorIndex = 0; colorIndex < activeColorCount; ++colorIndex )
+				{
+					syncBits = ( graphSyncIndex << 16 ) | iterStageIndex;
+					B2_ASSERT( stages[iterStageIndex].type == b2_stageSolve );
+					b2ExecuteMainStage( stages + iterStageIndex, context, syncBits );
+					iterStageIndex += 1;
+				}
+				graphSyncIndex += 1;
 			}
-			graphSyncIndex += 1;
 
 			profile->solveImpulses += b2GetMillisecondsAndReset( &ticks );
 
@@ -1045,24 +1050,27 @@ static void b2SolverTask( int startIndex, int endIndex, uint32_t threadIndexIgno
 
 			// relax constraints
 			useBias = false;
-			b2SolveOverflowJoints( context, useBias );
-			b2SolveOverflowContacts( context, useBias );
-
-			for ( int colorIndex = 0; colorIndex < activeColorCount; ++colorIndex )
+			for ( int j = 0; j < RELAX_ITERATIONS; ++j )
 			{
-				syncBits = ( graphSyncIndex << 16 ) | iterStageIndex;
-				B2_ASSERT( stages[iterStageIndex].type == b2_stageRelax );
-				b2ExecuteMainStage( stages + iterStageIndex, context, syncBits );
-				iterStageIndex += 1;
+				b2SolveOverflowJoints( context, useBias );
+				b2SolveOverflowContacts( context, useBias );
+
+				for ( int colorIndex = 0; colorIndex < activeColorCount; ++colorIndex )
+				{
+					syncBits = ( graphSyncIndex << 16 ) | iterStageIndex;
+					B2_ASSERT( stages[iterStageIndex].type == b2_stageRelax );
+					b2ExecuteMainStage( stages + iterStageIndex, context, syncBits );
+					iterStageIndex += 1;
+				}
+				graphSyncIndex += 1;
 			}
-			graphSyncIndex += 1;
 
 			profile->relaxImpulses += b2GetMillisecondsAndReset( &ticks );
 		}
 
 		// advance the stage according to the sub-stepping tasks just completed
 		// integrate velocities / warm start / solve / integrate positions / relax
-		stageIndex += 1 + activeColorCount + activeColorCount + 1 + activeColorCount;
+		stageIndex += 1 + activeColorCount + ITERATIONS * activeColorCount + 1 + RELAX_ITERATIONS * activeColorCount;
 
 		// Restitution
 		{
@@ -1442,11 +1450,11 @@ void b2Solve( b2World* world, b2StepContext* stepContext )
 		// b2_stageWarmStart
 		stageCount += activeColorCount;
 		// b2_stageSolve
-		stageCount += activeColorCount;
+		stageCount += ITERATIONS * activeColorCount;
 		// b2_stageIntegratePositions
 		stageCount += 1;
 		// b2_stageRelax
-		stageCount += activeColorCount;
+		stageCount += RELAX_ITERATIONS * activeColorCount;
 		// b2_stageRestitution
 		stageCount += activeColorCount;
 		// b2_stageStoreImpulses
@@ -1603,14 +1611,17 @@ void b2Solve( b2World* world, b2StepContext* stepContext )
 		}
 
 		// Solve graph
-		for ( int i = 0; i < activeColorCount; ++i )
+		for (int j = 0; j < ITERATIONS; ++j)
 		{
-			stage->type = b2_stageSolve;
-			stage->blocks = graphColorBlocks[i];
-			stage->blockCount = colorJointBlockCounts[i] + colorContactBlockCounts[i];
-			stage->colorIndex = activeColorIndices[i];
-			b2AtomicStoreInt( &stage->completionCount, 0 );
-			stage += 1;
+			for ( int i = 0; i < activeColorCount; ++i )
+			{
+				stage->type = b2_stageSolve;
+				stage->blocks = graphColorBlocks[i];
+				stage->blockCount = colorJointBlockCounts[i] + colorContactBlockCounts[i];
+				stage->colorIndex = activeColorIndices[i];
+				b2AtomicStoreInt( &stage->completionCount, 0 );
+				stage += 1;
+			}
 		}
 
 		// Integrate positions
@@ -1622,14 +1633,17 @@ void b2Solve( b2World* world, b2StepContext* stepContext )
 		stage += 1;
 
 		// Relax constraints
-		for ( int i = 0; i < activeColorCount; ++i )
+		for ( int j = 0; j < RELAX_ITERATIONS; ++j )
 		{
-			stage->type = b2_stageRelax;
-			stage->blocks = graphColorBlocks[i];
-			stage->blockCount = colorJointBlockCounts[i] + colorContactBlockCounts[i];
-			stage->colorIndex = activeColorIndices[i];
-			b2AtomicStoreInt( &stage->completionCount, 0 );
-			stage += 1;
+			for ( int i = 0; i < activeColorCount; ++i )
+			{
+				stage->type = b2_stageRelax;
+				stage->blocks = graphColorBlocks[i];
+				stage->blockCount = colorJointBlockCounts[i] + colorContactBlockCounts[i];
+				stage->colorIndex = activeColorIndices[i];
+				b2AtomicStoreInt( &stage->completionCount, 0 );
+				stage += 1;
+			}
 		}
 
 		// Restitution
