@@ -3,10 +3,10 @@
 
 #pragma once
 
-#include "box2d/math_functions.h"
-
 #include "core.h"
 
+#include "box2d/math_functions.h"
+
 #include <stdbool.h>
 #include <stdint.h>
 
@@ -86,7 +86,6 @@ typedef struct b2StepContext
 
 	int subStepCount;
 
-	b2Softness jointSoftness;
 	b2Softness contactSoftness;
 	b2Softness staticSoftness;
 
@@ -142,14 +141,44 @@ static inline b2Softness b2MakeSoft( float hertz, float zeta, float h )
 {
 	if ( hertz == 0.0f )
 	{
-		return ( b2Softness ){ 0.0f, 1.0f, 0.0f };
+		return (b2Softness){
+			.biasRate = 0.0f,
+			.massScale = 0.0f,
+			.impulseScale = 0.0f,
+		};
 	}
 
 	float omega = 2.0f * B2_PI * hertz;
 	float a1 = 2.0f * zeta + h * omega;
 	float a2 = h * omega * a1;
 	float a3 = 1.0f / ( 1.0f + a2 );
-	return ( b2Softness ){ omega / a1, a2 * a3, a3 };
+
+	// bias = w / (2 * z + hw)
+	// massScale = hw * (2 * z + hw) / (1 + hw * (2 * z + hw))
+	// impulseScale = 1 / (1 + hw * (2 * z + hw))
+
+	// If z == 0
+	// bias = 1/h
+	// massScale = hw^2 / (1 + hw^2)
+	// impulseScale = 1 / (1 + hw^2)
+
+	// w -> inf
+	// bias = 1/h
+	// massScale = 1
+	// impulseScale = 0
+
+	// if w = pi / 4  * inv_h
+	// massScale = (pi/4)^2 / (1 + (pi/4)^2) = pi^2 / (16 + pi^2) ~= 0.38
+	// impulseScale = 1 / (1 + (pi/4)^2) = 16 / (16 + pi^2) ~= 0.62
+
+	// In all cases:
+	// massScale + impulseScale == 1
+
+	return (b2Softness){
+		.biasRate = omega / a1,
+		.massScale = a2 * a3,
+		.impulseScale = a3,
+	};
 }
 
 void b2Solve( b2World* world, b2StepContext* stepContext );
